#!/usr/bin/python

from br_registrations.validateCPF import CPF as cpf_validator
import sqlite3
import random
import requests
import threading
import time

fetch_queue = []
write_queue = {}

fetch_queue_lock = threading.Lock()
write_queue_lock = threading.Lock()

def formatted_cpf(root: str) -> str:
    dvs = cpf_validator.get_cpf_dv(root)
    return root + str(dvs[0]) + str(dvs[1])

def random_cpf() -> str:
    root = '{0:09}'.format(random.randint(1, 999999999))
    return formatted_cpf(root)

def new_cpf() -> str:
    while True:
        with fetch_queue_lock:
            if fetch_queue:
                return fetch_queue.pop()
        time.sleep(0.1)

def threaded_cpf_fetch(id: int):
    session = requests.Session()
    while True:
        cpf = new_cpf()
        r = session.post('https://api.destinorastreios.online/api/consultar-cpf', json={"cpf":cpf})
        with write_queue_lock:
            if r.ok:
                write_queue[cpf] = r.text
            else:
                write_queue[cpf] = False
            
# sqlite3: create table cpf(cpf varchar2(11) primary key, contents text default null, ok bool default false);

if __name__ == "__main__":

    db = sqlite3.connect('/tmp/cpf.db')
    cur = db.cursor()
    
    fetch_threads = []
    
    for i in range(50):
        t = threading.Thread(target = threaded_cpf_fetch, args=(i,))
        fetch_threads.append(t)
        t.start()

    while True:
        with fetch_queue_lock:
            top_off = 10000
            while len(fetch_queue) < top_off:
                check_list = [random_cpf() for i in range(top_off - len(fetch_queue))]
                for cpf in check_list:
                    cur.execute("select cpf from cpf where cpf = '{0}'".format(cpf))
                    if not cur.fetchone():
                        if not cpf in fetch_queue:
                            fetch_queue.append(cpf)
            
        with write_queue_lock:
            while write_queue:
                cpf, contents = write_queue.popitem()
                print(cpf, end=' ... ')
                if contents:
                    p = {'cpf' : cpf, 'contents' : contents, 'ok' : True}
                    cur.execute('insert into cpf (cpf, contents, ok) values (:cpf, :contents, :ok)', p)
                    print('ok')
                else:
                    p = {'cpf' : cpf}
                    cur.execute('insert into cpf (cpf) values (:cpf)', p)
                    print('not found')
            db.commit()

        time.sleep(10)
        print(time.time())
